@@ -59,14 +59,14 @@ namespace HandBrakeWPF
 
         private void Init(StartupEventArgs e)
         {
-            if (!File.Exists("hb.dll") && !File.Exists(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "hb.dll")))
+            if (!SystemInfo.IsArmDevice && !File.Exists("hb.dll") && !File.Exists(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "hb.dll")))
             {
                 MessageBox.Show("hb.dll file not found. Application will not run correctly without this. Please re-install HandBrake.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                 this.Shutdown();
                 Environment.Exit(-1);
                 return;
             }
-
+            
             // We don't support Windows earlier than 10.
             if (!SystemInfo.IsWindows10OrLater())
             {
